
// Use Simple_Apu to play random tones. Write output to sound file "out.wav".

//#define TIMING
//#define RANDOM
//#define ENVELOPE
//#define SWEEP
//#define LENGTH_COUNTER
//#define NOISE
//#define TRIANGLE
//#define RECTANGLE

#define RECT_TEST

#include "Simple_Apu.h"

#include <stdlib.h>

// Uncomment to use SDL sound
//#include "SDL.h"

const long sample_rate = 44100;
static Simple_Apu apu;

// "emulate" 1/60 second of sound
static void emulate_frame()
{
	// Decay current tone
	static int volume;

	apu.write_register( 0x4000, 0xb0 | volume );
	volume--;
	if ( volume < 0 )
	{
		volume = 15;
		
		#ifdef RANDOM
		// Start a new random tone
		apu.write_register( 0x4015, 0x01 );
		apu.write_register( 0x4002, rand() & 0xff );
		apu.write_register( 0x4003, (rand() & 3) | 0x11 );
		
		#else
		// No random 
		apu.write_register( 0x4015, 0x01 );
		apu.write_register( 0x4002, 0xff );
		apu.write_register( 0x4003, 3 | 0x11 );	
		
		#endif
	}
	
	// Generate 1/60 second of sound into APU's sample buffer
	apu.end_frame();
}

static int read_dmc( void*, cpu_addr_t addr )
{
	// call your memory read function here
	//return read_memory( addr );
	return 0;
}

static void init_sound();
static void play_samples( const blip_sample_t*, long count );
static void cleanup_sound();

void timing_bench()
{
	// Generate a few seconds of sound
	for ( int n = 60 * 10; n--; )
	{
		// Simulate emulation of 1/60 second frame
		emulate_frame();
		
		// Samples from the frame can now be read out of the apu, or
		// allowed to accumulate and read out later. Use samples_avail()
		// to find out how many samples are currently in the buffer.
		
		int const buf_size = 2048;
		static blip_sample_t buf [buf_size];
		
		// Play whatever samples are available
		long count = apu.read_samples( buf, buf_size );
		play_samples( buf, count );
	}
}

void delay(int d)
{
	// Generate a few seconds of sound
	for ( int n = d; n--; )
	{
		// Simulate emulation of 1/60 second frame
		apu.end_frame();
		
		// Samples from the frame can now be read out of the apu, or
		// allowed to accumulate and read out later. Use samples_avail()
		// to find out how many samples are currently in the buffer.
		
		int const buf_size = 2048;
		static blip_sample_t buf [buf_size];
		
		// Play whatever samples are available
		long count = apu.read_samples( buf, buf_size );
		play_samples( buf, count );
	}
}

void micro_bench()
{

  #ifdef RECTANGLE
  
  //basic frequenc
  apu.write_register( 0x4000, 0x3f );
	apu.write_register( 0x4001, 0x00 );
	apu.write_register( 0x4002, 0x09 );
	apu.write_register( 0x4003, 0x00 );
	apu.write_register( 0x4015, 0x01 );	

	#endif

  #ifdef SWEEP
	apu.write_register( 0x4015, 0x01 );	
	apu.write_register( 0x4000, 0x3f);
	apu.write_register( 0x4001, 0xff);
	apu.write_register( 0x4002, 0xff );
	apu.write_register( 0x4003, 0xf1 );
	#endif
	
	#ifdef ENVELOPE
	apu.write_register( 0x4015, 0x01 );	
	apu.write_register( 0x4000, 0x20);
	apu.write_register( 0x4001, 0x00);
	apu.write_register( 0x4002, 0xff );
	apu.write_register( 0x4003, 0xf1 );
	#endif
	
	#ifdef LENGTH_COUNTER
	apu.write_register( 0x4015, 0x01 );	
	apu.write_register( 0x4000, 0x0f);
	apu.write_register( 0x4001, 0x00);
	apu.write_register( 0x4002, 0xff );
	apu.write_register( 0x4003, 0xf1 );
	#endif
	
	#ifdef TRIANGLE
	apu.write_register( 0x4015, 0x04 );	
	apu.write_register( 0x4008, 0x7f );
	apu.write_register( 0x400A, 0xff );
	apu.write_register( 0x400B, 0x7f );
	#endif
	
	#ifdef NOISE
	apu.write_register( 0x4015, 0x08 );	
	apu.write_register( 0x400C, 0x3f);
	apu.write_register( 0x400E, 0x8f);
	apu.write_register( 0x400F, 0xff );
	#endif
	
	
	#ifdef RECT_TEST
	
	//Enable rectangle1
  apu.write_register (0x4015, 0x01);

  //Simple tone - duty type 00
  apu.write_register( 0x4000, 0x3f );
	apu.write_register( 0x4001, 0x00 );
	apu.write_register( 0x4002, 0x09 );
	apu.write_register( 0x4003, 0x02 );

	
	//Delay
	delay(60);

  //Simple tone - duty type 01
  apu.write_register( 0x4000, 0x7f );
	apu.write_register( 0x4001, 0x00 );
	apu.write_register( 0x4002, 0x09 );
	apu.write_register( 0x4003, 0x02 );
	
	//Delay
	delay(60);
	
	//Simple tone - duty type 10
  apu.write_register( 0x4000, 0xbf );
	apu.write_register( 0x4001, 0x00 );
	apu.write_register( 0x4002, 0x09 );
	apu.write_register( 0x4003, 0x02 );
	
	//Delay
	delay(60);
	
  //Simple tone - duty type 11
  apu.write_register( 0x4000, 0xff );
	apu.write_register( 0x4001, 0x00 );
	apu.write_register( 0x4002, 0x09 );
	apu.write_register( 0x4003, 0x02 );
	
	//Delay
	delay(60);

  //Envelope decay
	apu.write_register( 0x4000, 0x29);
	apu.write_register( 0x4001, 0x00);
	apu.write_register( 0x4002, 0x09 );
	apu.write_register( 0x4003, 0x02 );
	
	//Delay
	delay(120);
	
	//Sweep
	apu.write_register( 0x4000, 0x3f);
	apu.write_register( 0x4001, 0xff);
	apu.write_register( 0x4002, 0xff);
	apu.write_register( 0x4003, 0xf1);
		
	//Delay
	delay(120);

	#endif
}



int main( int argc, char** argv )
{
	init_sound();
	
	// Set sample rate and check for out of memory error
	if ( apu.sample_rate( sample_rate ) )
		return EXIT_FAILURE;
	
	// Set function for APU to read memory with (required for DMC samples to play properly)
	apu.dmc_reader( read_dmc, NULL );
	
	
  #ifdef TIMING
	timing_bench();
	#else
	micro_bench();
	#endif
	
	cleanup_sound();
	
	return 0;
}


// Sound output handling (either to SDL or wave file)

#ifdef SDL_INIT_AUDIO

	#include "Sound_Queue.h"

	static Sound_Queue* sound_queue;
	
	static void init_sound()
	{
		if ( SDL_Init( SDL_INIT_AUDIO ) < 0 )
			exit( EXIT_FAILURE );
		
		atexit( SDL_Quit );
		
		sound_queue = new Sound_Queue;
		if ( !sound_queue )
			exit( EXIT_FAILURE );
		
		if ( sound_queue->init( sample_rate ) )
			exit( EXIT_FAILURE );
	}

	static void cleanup_sound()
	{
		delete sound_queue;
	}
	
	static void play_samples( const blip_sample_t* samples, long count )
	{
		sound_queue->write( samples, count );
	}
	
#else

	#include "Wave_Writer.hpp"
	
	static void init_sound()    { }
	static void cleanup_sound() { }

	static void play_samples( const blip_sample_t* samples, long count )
	{
		// write samples to sound file
		static Wave_Writer wave( sample_rate );
		wave.write( samples, count );
	}
	
#endif

