#include "APU_Analyze.h"

#include <stdio.h>
#include <malloc.h>
#include <math.h>

long long int APU_Analyze::cycle_counter;
struct instruction* APU_Analyze::instr_ptr;
struct memory_record* APU_Analyze::memory;
long long int APU_Analyze::instr_max_delay;
char APU_Analyze::instr_rect1;
char APU_Analyze::instr_rect2;
char APU_Analyze::instr_triangle;
char APU_Analyze::instr_noise;
char APU_Analyze::instr_dcm;
char APU_Analyze::instr_size;

void APU_Analyze::Clear()
{
  struct instruction *next;
  while (instr_ptr != NULL)
  {
    next = instr_ptr->next;
    free(instr_ptr);
    instr_ptr = next;
  }
}

void APU_Analyze::Print()
{

  printf("//Auto-generated file\n\n");
  printf("//STATISTICS\n\n");
  printf ("//Used Channels:\n");
  if(instr_rect1 > 0) printf("//Rectangle 1\n");
  if(instr_rect2 > 0) printf("//Rectangle 2\n");
  if(instr_triangle > 0) printf("//Triangle\n");
  if(instr_noise > 0) printf("//Noise\n");
  if(instr_dcm > 0) printf("//DCM\n");

  printf("\n\n");

  struct instruction* tmp = instr_ptr;
  long long int time1 = 0;
  long long int time2 = 0;
  long long int max_delay = -1;
  long long int total = 0;

  while (tmp != NULL)
  {
    total ++;
    time2 = tmp->time;
    tmp->time = (tmp->time)-time1;
    time1 = time2;  
    if (tmp->time > max_delay) max_delay= tmp->time;
    tmp = tmp->next;
  }
    
  double d1 = log((double) max_delay);
  double d2 = log(2.0);
  double d3 = ceil(d1/d2)/8;
  int delay_size = ceil(d3);
  
  if (delay_size > 4) {
  printf("DELAY SIZE PROBLEM, BIGGER THAN 4 BYTES!!!;");
  printf("Aborting.\n");
  return;
  }
  
  printf("#define NUM_OF_INSTRUCTIONS %lld\n\n", total);
  
  double x = (cycle_counter*10.0)/NUM_OF_SECONDS;
  
  double sf = 7*1000000*pow(x, -0.96345); //-0.96
  
  unsigned long long int slow_factor = sf;
        
  printf("#define SLOW_FACTOR %lld\n\n", slow_factor);
  
  long long int dcm_size = 0;
  
  //DCM MEMORY
  struct memory_record* tmp1  = memory;
  while (tmp1 != NULL)
  {
    dcm_size++;
    tmp1 = tmp1->next;
  }
  
  printf("#define DCM_MEM_SIZE %lld\n\n", dcm_size);
  
  
  printf("static const unsigned int dcm_address[DCM_MEM_SIZE] = {\n  ");
  int counter = 0;
  tmp1  = memory;
  while (tmp1 != NULL)
  {
    if (counter == 20) {printf("\n  "); counter = 0;}
    printf("%d", tmp1->addr - 0x8000);
    if (tmp1->next != NULL) printf(", ");
    tmp1 = tmp1->next;
    counter++;
  }
  printf("};\n\n");
  
  
  printf("static const unsigned char dcm_data[DCM_MEM_SIZE] = {\n  ");
   counter = 0;
  tmp1  = memory;
  while (tmp1 != NULL)
  {
    if (counter == 20) {printf("\n  "); counter = 0;}
    printf("%d", tmp1->data);
    if (tmp1->next != NULL) printf(", ");
    tmp1 = tmp1->next;
    counter++;
  }
  printf("};\n\n");
  
  

  
  //ADDRESSES
  printf("static const unsigned char address[NUM_OF_INSTRUCTIONS] = {\n  ");
   counter = 0;
  tmp = instr_ptr;
  while (tmp != NULL)
  {
    if (counter == 20) {printf("\n  "); counter = 0;}
    printf("%d", tmp->addr - 0x4000);
    if (tmp->next != NULL) printf(", ");
    tmp = tmp->next;
    counter++;
  }
  printf("};\n\n");
  
  //DATA
  printf("static const unsigned char data[NUM_OF_INSTRUCTIONS] = {\n  ");
  counter = 0;
  tmp = instr_ptr;
  while (tmp != NULL)
  {
    //printf("%x %02x\n", tmp->addr, tmp->data);
    if (counter == 20) {printf("\n  "); counter = 0;}
    printf("%d", tmp->data);
    if (tmp->next != NULL) printf(", ");
    tmp = tmp->next;
    counter++;
  }
  printf("};\n\n");
    
  //DELAYS  
  if (delay_size == 1)
  {
		printf("static const unsigned short int delay[NUM_OF_INSTRUCTIONS] = {\n  ");
	}
  if (delay_size == 2)
  {
		printf("static const unsigned short int delay[NUM_OF_INSTRUCTIONS] = {\n  ");
	}
	if (delay_size > 2)
  {
		printf("static const unsigned int delay[NUM_OF_INSTRUCTIONS] = {\n  ");
	} 
	
  counter = 0;
  tmp = instr_ptr;
  while (tmp != NULL)
  {
    if (counter == 20) {printf("\n  "); counter = 0;}
    printf("%d", tmp->time);
    if (tmp->next != NULL) printf(", ");
    tmp = tmp->next;
    counter++;
  }
  printf("};\n\n"); 
}


int APU_Analyze::Add(int addr, int *data)
{

    if (addr == 0x4015)
    {
  
      #ifdef IGNORE_RECTANGLE1
      *data = *data & 0xfe;
      #endif
      
      #ifdef IGNORE_RECTANGLE2
      *data = *data & 0xfd;
      #endif
 
      #ifdef IGNORE_TRIANGLE
      *data = *data & 0xfb;
      #endif
 
      #ifdef IGNORE_NOISE
      *data = *data & 0xf7;
      #endif
 
      #ifdef IGNORE_DCM
      *data = *data & 0xef;
      #endif
    }

  if (cycle_counter > 0)
  {
    if ((addr >= 0x4000) && (addr <= 0x4003))
    {
      #ifdef IGNORE_RECTANGLE1
      return 1;
      #endif
      instr_rect1 = 1;
    }
		if ((addr >= 0x4004) && (addr <= 0x4007))
		{ 
		  #ifdef IGNORE_RECTANGLE2
      return 1;
      #endif
		  instr_rect2 = 1;
	  }
		if ((addr >= 0x4008) && (addr <= 0x400b))
		{
		  #ifdef IGNORE_TRIANGLE
      return 1;
      #endif
		  instr_triangle = 1;
	  } 
		if ((addr >= 0x400c) && (addr <= 0x400f))
		{
		  #ifdef IGNORE_NOISE
      return 1;
      #endif
		  instr_noise = 1;
	  } 
		if ((addr == 0x4010) && (addr <= 0x4013))
		{
		 // if (addr != 0x4012) return 1;
		  #ifdef IGNORE_DCM
      return 1;
      #endif
		  instr_dcm = 1;
    }
    

    
  } else
  {
    return 0;
  }
  

  struct instruction* tmp = instr_ptr;
  
  if (instr_ptr == NULL)
  {
    instr_ptr = (struct instruction*) malloc (sizeof(struct instruction));
    instr_ptr->addr = addr;
    instr_ptr->data = *data;
    instr_ptr->time = cycle_counter;
    instr_ptr->next = NULL;
  } else
  {
  
  while(tmp->next != NULL) tmp = tmp->next;

  tmp->next = (struct instruction*) malloc (sizeof(struct instruction));
  tmp->next->addr = addr;
  tmp->next->data = *data;
  tmp->next->time = cycle_counter;
  tmp->next->next = NULL;
  }
  
  return 0;
} 


void APU_Analyze::RecordMemory(int address, int data)
{
  struct memory_record* tmp = memory;
  
  if (memory == NULL)
  {
    memory = (struct memory_record*) malloc (sizeof(struct memory_record));
    memory->addr = address;
    memory->data = data;
    memory->next = NULL;
  } else
  
  {
  
  while(tmp->next != NULL) 
  {
    if ((tmp->data == data) && (tmp->addr == address) ) return;
    tmp = tmp->next;
  }

  tmp->next = (struct memory_record*) malloc (sizeof(struct memory_record));
  tmp->next->addr = address;
  tmp->next->data = data;
  tmp->next->next = NULL;
  
  }
}
  